;RNG DLL Template for use with RNG.exe.
;The functions RandomIntArray and InitRand32 must be exported, so edit the DEF 
;file accordingly.  The three procedures may be modified as you will, bearing in 
;mind that their I/O must match requirements of RNG.exe.

;***** Export File RandTemplate.def *****
;LIBRARY RandTemplate
;EXPORTS RandomIntArray
;EXPORTS InitRand32
;EXPORTS Rand32 (not required for RNG.exe))
;****************************************

.NOLIST
.586
.model flat,stdcall
option casemap:none
include	\masm32\include\windows.inc

Rand32		proto	:DWORD
InitRand32	proto   :DWORD

.data?
seed	dd	?

.code
DllEntry	proc	hinstance:HINSTANCE, reason:DWORD, reserved1:DWORD
		invoke  InitRand32,0
		mov	eax,TRUE
		ret
DllEntry	endp

;Usage: RandomIntArray(Data:pointer,count:int,base:int,seed:int),int
;Returns a series of random integers, base>n>=0, to a dword array.  sede is not
;used with RNG.exe, and is always -1.
RandomIntArray	proc uses esi pData:PTR DWORD, count:DWORD, base:DWORD, sede:dword
;	invoke  InitRand32,sede
	mov     esi,pData
	mov     ecx,count
	mov     edx,base
@@:	or      ecx,ecx
	jz      @F
	push    ecx
	push    edx
	invoke  Rand32,edx
	pop     edx
	pop     ecx
	mov     [esi],eax
	add     esi,4
	dec     ecx
	jmp     @B
@@:	ret
RandomIntArray  endp

;Usage: InitRand32(seed:int)
;Sets a starting seed for random number generation.  An input value of zero 
;uses the cpu time-stamp clock as a 'randomize' option.  Negative seeds 
;are ignored, allowing queries without changes.  The function returns 
;the current seed.  RNG.exe calls this function to set the initial seed 
;for a series of runs and queries this function (seed=-1) to find the seed 
;at the start of each run for display purposes.
;
InitRand32	proc	new_seed:DWORD
	mov	eax,new_seed
	test	eax,eax
	jnz	ir01		;If non-zero seed specified
@@:	rdtsc			; else get value from CPU clock
	and	eax,7fffffffh	; and clear msbit
	jz	@B		;IF TSC seed just happens to be 0 then retry

ir01:	cmp	eax,7fffffffh	;If seed > 7ffffffe, no changes
	jb	@F
	mov	eax,seed
@@:	mov	seed,eax
	cmp	eax,new_seed	;set flags returned for possible testing
	ret			; and return seed value
InitRand32	endp
;Usage: Rand32(base:int)
;Returns a number ranging from 0 to base-1.  Valid base values range 1 
;through 7fffffff.  If the requested base lies outside this range, 
; -1 is returned, indicating the error.
;
Rand32	proc	base:DWORD
	mov	eax,base		;valid values 1 thru 7fffffff
	test	eax,eax
	jle	@F			;skip if base <= 0 (invalid)
	mov	eax,seed    
	mov	ecx,16807		;a = 7^5
	mul	ecx			;edx:eax == a*seed
	mov	ecx,7fffffffh
;	div	ecx
;	mov	eax,edx			;(a*seed) % 7fffffff
	add	edx,edx
        cmp     eax,ecx
        jna     @F
        sub     eax,ecx
@@:	add	eax,edx
	jns	@F
	sub	eax,ecx
@@:	mov	seed,eax		;save next seed 

;Using LoBits
	xor	edx,edx
	div	base

;Using HiBits
;	shl	eax,1
;	mul	base

	mov	eax,edx
  	ret

@@:	or	eax,-1			;if base error, return -1
	ret				; and set S flag.
Rand32	endp

End DllEntry
